package sbbcommandline;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.zip.GZIPInputStream;
import org.apache.commons.io.FileUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.BasicHttpEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.jdom2.Content;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import sbbcommandline.objects.SBBStation;

public class SBBCommandLine {
    
    private final static String BASE_URL = "http://xmlfahrplan.sbb.ch/bin/extxml.exe/";
    private final static String USER_AGENT = "SBBMobile/%1$s (Android/%2$s %3$s; SDK/%4$s; fr)";
    
    private final static String STATION_QUERY = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n"
            + "<ReqC lang=\"EN\" prod=\"unknown alien_jolla_bionic, Android OS: 4.1.2 (SDK:16) \" ver=\"3.0.5\" accessId=\"sbuR5gsaDCEzWpCzSuEnxJOmDbb5vEM1vf0tWwjY3s6eab2uj8dAiLvjisGKDnN\">\n"
            + "  <LocValReq id=\"START\" maxNr=\"25\">\n"
            + "    <ReqLoc match=\"%1$s\" type=\"ST\" />\n"
            + "  </LocValReq>\n"
            + "</ReqC>";
    
    private final static String CONNECTION_QUERY = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n"
            + "<ReqC lang=\"EN\" prod=\"unknown alien_jolla_bionic, Android OS: 4.1.2 (SDK:16) \" ver=\"3.0.5\" accessId=\"sbuR5gsaDCEzWpCzSuEnxJOmDbb5vEM1vf0tWwjY3s6eab2uj8dAiLvjisGKDnN\">\n"
            + "  <ConReq>\n"
            + "    <Start>\n"
            + "      <Station name=\"%1$s\" externalId=\"%2$s\" />\n"
            + "      <Prod prod=\"1111111111000000\" />\n"
            + "    </Start>\n"
            + "    <Dest>\n"
            + "      <Station name=\"%3$s\" externalId=\"%4$s\" />\n"
            + "    </Dest>\n"
            + "    <ReqT a=\"0\" date=\"%5$s\" time=\"%6$s\" />\n"
            + "    <RFlags b=\"0\" f=\"6\" sMode=\"N\" />\n"
            + "  </ConReq>\n"
            + "</ReqC>";

    public static void main(String[] args) throws IOException, JDOMException {
        
        SBBStation startStation = getStation("Icogne poste");
        SBBStation endStation = getStation("Lausanne jordils");
        
        getConnections(startStation, endStation, new Date());
    }
    
    private static SBBStation getStation(String stationName) throws IOException, JDOMException {
        String request = String.format(STATION_QUERY, stationName);
        
        Element rootElement = getResponse(request);
        Element stationListElement = rootElement.getChild("LocValRes");
        
        Element stationElement = stationListElement.getChild("Station");
        
        SBBStation startStation = new SBBStation(stationName);
        startStation.fillFromXML(stationElement);
        
        return startStation;
        
    }
    
    private static Element getResponse(String requestXML) throws IOException, JDOMException {
        byte[] requestBytes = requestXML.getBytes();
        String response = new SBBCommandLine().testDL(requestBytes);
        
        System.out.println("Got response " + response);
        
        Document doc = new SAXBuilder().build(new StringReader(response));
        
        //ConRes is the root element of the response.
        Element rootElement = doc.getRootElement();
        
        return rootElement;
    }
    
    private static void getConnections(SBBStation start, SBBStation end, Date departure) throws IOException, JDOMException {
        
        String date = new SimpleDateFormat("yyyyMMdd").format(departure);
        String time = new SimpleDateFormat("HH:mm").format(departure);
        
        String request = String.format(CONNECTION_QUERY, start.getName(), start.getExternalId(), end.getName(), end.getExternalId(), date, time);
        
        System.out.println("Request is " + request);
        //if (request != null) return;
        
        Element rootElement = getResponse(request);
        
        Element conResElement = rootElement.getChild("ConRes");
        
        if (conResElement.getChild("Err") != null) {
            // We encountered an error, print it out and abort, no more info is included
            // in the response.
            
            Element errorElem = conResElement.getChild("Err");
            System.out.println("Got error code (" + errorElem.getAttributeValue("code") + "): " + errorElem.getAttributeValue("text"));
            return;
        }
        
        //ConResCtxt is a "Cookie" that has to be sent if we want to "scroll".
        Element conResCtxtElement = conResElement.getChild("ConResCtxt");
        
        // ConnectionList contains the list of connections
        Element connectionListElement = conResElement.getChild("ConnectionList");
        for (Element elem : connectionListElement.getChildren()) {
            dumpConnection(elem);
        }
    }
    
    private static void dumpConnection(Element connectionElement) {
        Element overviewElement = connectionElement.getChild("Overview");
        Element conSectionListElement = connectionElement.getChild("ConSectionList");
        
        String time = overviewElement.getChild("Departure").getChild("BasicStop").getChild("Dep").getChild("Time").getValue();
        String station = overviewElement.getChild("Departure").getChild("BasicStop").getChild("Station").getAttributeValue("name");
        
        System.out.println();
        System.out.println();
        System.out.println("Departure from " + station + " at " + time);
        
        for (Element elem : conSectionListElement.getChildren()) {
            
            Element departureElement = elem.getChild("Departure");
            Element arrivalElement = elem.getChild("Arrival");
            
            String depName = departureElement.getChild("BasicStop").getChild("Station").getAttributeValue("name");
            String arrName = arrivalElement.getChild("BasicStop").getChild("Station").getAttributeValue("name");
            System.out.println("From " + depName + " to " + arrName);
        }
        
    }
    
    
    private static String getUserAgent() {
        String str = USER_AGENT;
        Object[] arrayOfObject = new Object[4];
        arrayOfObject[0] = "2.3.2";
        arrayOfObject[1] = "4.1.2";
        arrayOfObject[2] = "Fusion Cookie";
        arrayOfObject[3] = "12";
        return String.format(str, arrayOfObject);
    }

    private String testDL(byte[] paramArrayOfByte) throws IOException {
        DefaultHttpClient localDefaultHttpClient = new DefaultHttpClient();
        HttpPost localHttpPost = new HttpPost(BASE_URL);
        BasicHttpEntity localBasicHttpEntity = new BasicHttpEntity();
        localBasicHttpEntity.setContentType("application/xml");
        ByteArrayInputStream localByteArrayInputStream = new ByteArrayInputStream(paramArrayOfByte);
        localBasicHttpEntity.setContent(localByteArrayInputStream);
        localBasicHttpEntity.setContentLength(paramArrayOfByte.length);
        localHttpPost.setEntity(localBasicHttpEntity);
        
        localHttpPost.setHeader("User-Agent", getUserAgent());
        
        localHttpPost.setHeader("Accept", "application/xml");
        
        localHttpPost.setHeader("Accept-Language", "FR");
        
        localHttpPost.getParams().setBooleanParameter("http.protocol.expect-continue", false);
        localHttpPost.setHeader("Accept-Encoding", "gzip");
        
        String str1 = null; // No cookies ...
        if (str1 != null) {
            localHttpPost.setHeader("Cookie", "AL_LB=" + str1);
        }
        
        // No redirect ?
        //localDefaultHttpClient.setRedirectHandler(EnvironmentUtil.getRedirectHandler());
        HttpResponse localHttpResponse = localDefaultHttpClient.execute(localHttpPost);
        int i = localHttpResponse.getStatusLine().getStatusCode();
        
        InputStream content = localHttpResponse.getEntity().getContent();
        HttpEntity localHttpEntity = localHttpResponse.getEntity();
        
        //System.out.println("Got response status " + i + " of size " + localBasicHttpEntity.getContentLength() + " bytes.");
        
        byte[] arrayOfByte = new byte[1024];
        InputStream is;
        if ((localHttpEntity.getContentEncoding() != null) && ("gzip".equals(localHttpEntity.getContentEncoding().getValue()))) {
            is = new GZIPInputStream(localHttpResponse.getEntity().getContent());
        } else {
            is = new BufferedInputStream(localHttpResponse.getEntity().getContent());
        }
        
        OutputStream baos = new ByteArrayOutputStream();
        while (true) {
            int k = is.read(arrayOfByte, 0, 1024);
            if (k == -1) {
                break;
            }
            baos.write(arrayOfByte, 0, k);
        }
        
        return ((ByteArrayOutputStream) baos).toString();

    }

}
